.MODEL SMALL
.486

; 定义端口地址
I8255_A EQU 210H       ; 开关输入端口（PA）
I8255_B EQU 211H       ; LED输出端口（PB）
I8255_CTR EQU 213H     ; 控制寄存器
I0832_CS EQU 220H      ; D/A转换器端口

; 数据段
DATA SEGMENT
     SIN DB 80H,96H,0AEH,0C5H,0D8H,0E9H,0F5H,0FDH
        DB 0FFH,0FDH,0F5H,0E9H,0D8H,0C5H,0AEH,96H
        DB 80H,66H,4EH,38H,25H,15H,09H,04H
        DB 00H,04H,09H,15H,25H,38H,4EH,66H   ; 正弦波数据

    WAVE_FLAG DB 0     ; 波形标志位（0：无波形，1：正弦波，2：三角波，3：锯齿波，4：梯形波，5：方波）
    DUTY_CYCLE DB 0    ; 方波占空比（0：无输出，1，2，3）
    LED_STATE DB 0     ; PB端口状态
DATA ENDS

; 堆栈段
STACK SEGMENT STACK
    DB 100H DUP(0)     ; 分配256字节的堆栈空间
STACK ENDS

; 代码段
CODE SEGMENT
    ASSUME CS:CODE, DS:DATA, SS:STACK

START:
    ; 初始化数据段
    MOV AX, DATA
    MOV DS, AX

    ; 初始化堆栈段
    MOV AX, STACK
    MOV SS, AX
    MOV SP, OFFSET STACK + 100H  ; 设置堆栈指针到堆栈段的末尾

    ; 初始化8255，端口A为输入，端口B为输出
    MOV DX, I8255_CTR
    MOV AL, 90H         ; 控制字：端口A为输入，端口B为输出
    OUT DX, AL

    ; 初始化WAVE_FLAG为0（无波形）
    MOV WAVE_FLAG, 0

    ; 调用主程序
    CALL MAIN_LOOP

    ; 程序结束
    MOV AH, 4CH         ; 退出程序
    INT 21H

; 主程序
MAIN_LOOP PROC
    ; 读取PA端口状态（按键状态）
    MOV DX, I8255_A
    IN AL, DX

    ; 检测K1~K8的状态，并设置PB1~PB8的输出
    TEST AL, 01H        ; 检测K1
    JNZ SET_PB1
    TEST AL, 02H        ; 检测K2
    JNZ SET_PB2
    TEST AL, 04H        ; 检测K3
    JNZ SET_PB3
    TEST AL, 08H        ; 检测K4
    JNZ SET_PB4
    TEST AL, 10H        ; 检测K5
    JNZ SET_PB5
    TEST AL, 20H        ; 检测K6
    JNZ SET_PB6
    TEST AL, 40H        ; 检测K7
    JNZ SET_PB7
    TEST AL, 80H        ; 检测K8
    JNZ SET_PB8

    ; 如果没有按键按下，关闭所有LED
    MOV LED_STATE, 00H
    JMP UPDATE_LED

; 设置PB1为高电平
SET_PB1:
    MOV LED_STATE, 01H  ; PB1 = 1
    JMP UPDATE_LED

; 设置PB2为高电平
SET_PB2:
    MOV LED_STATE, 02H  ; PB2 = 1
    JMP UPDATE_LED

; 设置PB3为高电平
SET_PB3:
    MOV LED_STATE, 04H  ; PB3 = 1
    JMP UPDATE_LED

; 设置PB4为高电平
SET_PB4:
    MOV LED_STATE, 08H  ; PB4 = 1
    JMP UPDATE_LED

; 设置PB5为高电平
SET_PB5:
    MOV LED_STATE, 10H  ; PB5 = 1
    JMP UPDATE_LED

; 设置PB6为高电平
SET_PB6:
    MOV LED_STATE, 20H  ; PB6 = 1
    JMP UPDATE_LED

; 设置PB7为高电平
SET_PB7:
    MOV LED_STATE, 40H  ; PB7 = 1
    JMP UPDATE_LED

; 设置PB8为高电平
SET_PB8:
    MOV LED_STATE, 80H  ; PB8 = 1
    JMP UPDATE_LED

; 更新PB端口状态
UPDATE_LED:
    MOV DX, I8255_B
    MOV AL, LED_STATE
    OUT DX, AL

    ; 检测K0~K4的状态（优先级：K0 > K1 > K2 > K3 > K4）
    MOV DX, I8255_A
    IN AL, DX
    TEST AL, 01H        ; 检测K0
    JNZ SET_SIN_WAVE
    TEST AL, 02H        ; 检测K1
    JNZ SET_TRIANGLE_WAVE
    TEST AL, 04H        ; 检测K2
    JNZ SET_SAWTOOTH_WAVE
    TEST AL, 08H        ; 检测K3
    JNZ SET_TRAPEZOID_WAVE
    TEST AL, 10H        ; 检测K4
    JNZ SET_SQUARE_WAVE

    ; 如果K0~K4均未按下，关闭波形输出
    MOV WAVE_FLAG, 0
    JMP GENERATE_WAVE

; 设置正弦波生成标志位
SET_SIN_WAVE:
    MOV WAVE_FLAG, 1
    JMP GENERATE_WAVE

; 设置三角波生成标志位
SET_TRIANGLE_WAVE:
    MOV WAVE_FLAG, 2
    JMP GENERATE_WAVE

; 设置锯齿波生成标志位
SET_SAWTOOTH_WAVE:
    MOV WAVE_FLAG, 3
    JMP GENERATE_WAVE

; 设置梯形波生成标志位
SET_TRAPEZOID_WAVE:
    MOV WAVE_FLAG, 4
    JMP GENERATE_WAVE

; 设置方波生成标志位
SET_SQUARE_WAVE:
    ; 如果K0~K3未按下，检测K5~K7的状态
    MOV WAVE_FLAG, 5
    MOV DUTY_CYCLE, 0   ; 默认无输出

    ; 检测K5~K7的状态
    TEST AL, 20H        ; 检测K5
    JNZ SET_DUTY_25
    TEST AL, 40H        ; 检测K6
    JNZ SET_DUTY_50
    TEST AL, 80H        ; 检测K7
    JNZ SET_DUTY_75

    ; 如果K5~K7中有多个按键按下，关闭方波输出
    MOV WAVE_FLAG, 0
    JMP GENERATE_WAVE

SET_DUTY_25:
    MOV DUTY_CYCLE, 1   ; 占空比1
    JMP GENERATE_WAVE

SET_DUTY_50:
    MOV DUTY_CYCLE, 2   ; 占空比2
    JMP GENERATE_WAVE

SET_DUTY_75:
    MOV DUTY_CYCLE, 3   ; 占空比3
    JMP GENERATE_WAVE

; 波形生成逻辑
GENERATE_WAVE:
    ; 根据波形标志位生成对应的波形
    CMP WAVE_FLAG, 1
    JE CALL_SIN_WAVE
    CMP WAVE_FLAG, 2
    JE CALL_TRIANGLE_WAVE
    CMP WAVE_FLAG, 3
    JE CALL_SAWTOOTH_WAVE
    CMP WAVE_FLAG, 4
    JE CALL_TRAPEZOID_WAVE
    CMP WAVE_FLAG, 5
    JE CALL_SQUARE_WAVE

    ; 无波形输出
    MOV DX, I0832_CS
    MOV AL, 0           ; 低电平
    OUT DX, AL
    JMP MAIN_LOOP

; 调用正弦波生成子程序
CALL_SIN_WAVE:
    CALL SIN_WAVE
    JMP MAIN_LOOP

; 正弦波生成子程序
SIN_WAVE:
    MOV SI, OFFSET SIN  ; 正弦波数据地址
    MOV CX, 32          ; 32个数据点
SIN_LOOP:
    MOV AL, [SI]        ; 读取数据
    MOV DX, I0832_CS
    OUT DX, AL          ; 输出到D/A转换器
    INC SI              ; 下一个数据点
    LOOP SIN_LOOP       ; 循环，直到所有数据点输出完毕
    RET                 ; 返回调用点

; 调用三角波生成子程序
CALL_TRIANGLE_WAVE:
    CALL TRIANGLE_WAVE
    JMP MAIN_LOOP

; 三角波生成子程序
TRIANGLE_WAVE:
    MOV AL, 0           ; 初始值为0
TRIANGLE_UP:
    MOV DX, I0832_CS
    OUT DX, AL
    INC AL              ; 递增
    CMP AL, 0FFH        ; 判断是否到顶
    JNE TRIANGLE_UP
TRIANGLE_DOWN:
    MOV DX, I0832_CS
    OUT DX, AL
    DEC AL              ; 递减
    CMP AL, 0           ; 判断是否到底
    JNE TRIANGLE_DOWN
    RET                 ; 返回调用点

; 调用锯齿波生成子程序
CALL_SAWTOOTH_WAVE:
    CALL SAWTOOTH_WAVE
    JMP MAIN_LOOP

; 锯齿波生成子程序
SAWTOOTH_WAVE:
    MOV AL, 0           ; 初始值为0
SAWTOOTH_LOOP:
    MOV DX, I0832_CS
    OUT DX, AL
    INC AL              ; 递增
    CMP AL, 0FFH        ; 判断是否到顶
    JNE SAWTOOTH_LOOP
    RET                 ; 返回调用点

; 调用梯形波生成子程序
CALL_TRAPEZOID_WAVE:
    CALL TRAPEZOID_WAVE
    JMP MAIN_LOOP

; 梯形波生成子程序
TRAPEZOID_WAVE:
    MOV AL, 0           ; 初始值为0
TRAPEZOID_UP:
    MOV DX, I0832_CS
    OUT DX, AL
    INC AL              ; 递增
    CMP AL, 0FFH        ; 判断是否到顶
    JNE TRAPEZOID_UP
    MOV CX, 0FFH        ; 延时
TRAPEZOID_DELAY:
    LOOP TRAPEZOID_DELAY
TRAPEZOID_DOWN:
    MOV DX, I0832_CS
    OUT DX, AL
    DEC AL              ; 递减
    CMP AL, 0           ; 判断是否到底
    JNE TRAPEZOID_DOWN
    RET                 ; 返回调用点

; 调用方波生成子程序
CALL_SQUARE_WAVE:
    CALL SQUARE_WAVE
    JMP MAIN_LOOP

; 方波生成子程序
SQUARE_WAVE:
    ; 根据占空比生成方波
    CMP DUTY_CYCLE, 1
    JE DUTY_25
    CMP DUTY_CYCLE, 2
    JE DUTY_50
    CMP DUTY_CYCLE, 3
    JE DUTY_75

    ; 无输出
    MOV DX, I0832_CS
    MOV AL, 0           ; 低电平
    OUT DX, AL
    RET                 ; 返回调用点

DUTY_25:
    MOV BX, 64          ; 高电平时间占空比1（23.3%）
        JMP OUTPUT_WAVE

DUTY_50:
    MOV BX, 130         ; 高电平时间占空比2（45%）
    JMP OUTPUT_WAVE

DUTY_75:
    MOV BX, 200         ; 高电平时间占空比3（70%）
    JMP OUTPUT_WAVE

OUTPUT_WAVE:
    ; 保存BX的值
    PUSH BX

  

    ; 输出高电平
    MOV DX, I0832_CS
    MOV AL, 0FFH        ; 高电平
    OUT DX, AL          ; 将AL的值输出到D/A转换器

    ; 设置高电平延时
    MOV CX, BX          ; 将高电平时间赋值给CX
HIGH_DELAY:
    LOOP HIGH_DELAY    ; 延时

    ; 输出低电平
    MOV AL, 0           ; 低电平
    OUT DX, AL          ; 将AL的值输出到D/A转换器
  
; 计算低电平时间
    MOV AX, 255
    SUB AX, BX          ; 低电平时间 = 255 - 高电平时间

    ; 设置低电平延时
    MOV CX, AX          ; 将低电平时间赋值给CX
LOW_DELAY:
    LOOP LOW_DELAY     ; 延时

    ; 恢复BX的值
    POP BX

    RET                 ; 返回调用点

MAIN_LOOP ENDP

CODE ENDS
    END START